@@ -81,7 +81,7 @@ class Cache {
 			if(h.data != null) System.arraycopy(h.data,0,new_data,0,h.len);
 			h.data = new_data;
 			size -= more;
-			do {int _=h.len; h.len=len; len=_;} while(false);
+			do {int tmp=h.len; h.len=len; len=tmp;} while(false);
 		}
 
 		lru_insert(h);
@@ -95,18 +95,18 @@ class Cache {
 		
 		if(head[i].len > 0) lru_delete(head[i]);
 		if(head[j].len > 0) lru_delete(head[j]);
-		do {float[] _=head[i].data; head[i].data=head[j].data; head[j].data=_;} while(false);
-		do {int _=head[i].len; head[i].len=head[j].len; head[j].len=_;} while(false);
+		do {float[] tmp=head[i].data; head[i].data=head[j].data; head[j].data=tmp;} while(false);
+		do {int tmp=head[i].len; head[i].len=head[j].len; head[j].len=tmp;} while(false);
 		if(head[i].len > 0) lru_insert(head[i]);
 		if(head[j].len > 0) lru_insert(head[j]);
 
-		if(i>j) do {int _=i; i=j; j=_;} while(false);
+		if(i>j) do {int tmp=i; i=j; j=tmp;} while(false);
 		for(head_t h = lru_head.next; h!=lru_head; h=h.next)
 		{
 			if(h.len > i)
 			{
 				if(h.len > j)
-					do {float _=h.data[i]; h.data[i]=h.data[j]; h.data[j]=_;} while(false);
+					do {float tmp=h.data[i]; h.data[i]=h.data[j]; h.data[j]=tmp;} while(false);
 				else
 				{
 					// give up
@@ -148,8 +148,8 @@ abstract class Kernel extends QMatrix {
 
 	void swap_index(int i, int j)
 	{
-		do {svm_node[] _=x[i]; x[i]=x[j]; x[j]=_;} while(false);
-		if(x_square != null) do {double _=x_square[i]; x_square[i]=x_square[j]; x_square[j]=_;} while(false);
+		do {svm_node[] tmp=x[i]; x[i]=x[j]; x[j]=tmp;} while(false);
+		if(x_square != null) do {double tmp=x_square[i]; x_square[i]=x_square[j]; x_square[j]=tmp;} while(false);
 	}
 
 	private static double powi(double base, int times)
@@ -350,13 +350,13 @@ class Solver {
 	void swap_index(int i, int j)
 	{
 		Q.swap_index(i,j);
-		do {byte _=y[i]; y[i]=y[j]; y[j]=_;} while(false);
-		do {double _=G[i]; G[i]=G[j]; G[j]=_;} while(false);
-		do {byte _=alpha_status[i]; alpha_status[i]=alpha_status[j]; alpha_status[j]=_;} while(false);
-		do {double _=alpha[i]; alpha[i]=alpha[j]; alpha[j]=_;} while(false);
-		do {double _=p[i]; p[i]=p[j]; p[j]=_;} while(false);
-		do {int _=active_set[i]; active_set[i]=active_set[j]; active_set[j]=_;} while(false);
-		do {double _=G_bar[i]; G_bar[i]=G_bar[j]; G_bar[j]=_;} while(false);
+		do {byte tmp=y[i]; y[i]=y[j]; y[j]=tmp;} while(false);
+		do {double tmp=G[i]; G[i]=G[j]; G[j]=tmp;} while(false);
+		do {byte tmp=alpha_status[i]; alpha_status[i]=alpha_status[j]; alpha_status[j]=tmp;} while(false);
+		do {double tmp=alpha[i]; alpha[i]=alpha[j]; alpha[j]=tmp;} while(false);
+		do {double tmp=p[i]; p[i]=p[j]; p[j]=tmp;} while(false);
+		do {int tmp=active_set[i]; active_set[i]=active_set[j]; active_set[j]=tmp;} while(false);
+		do {double tmp=G_bar[i]; G_bar[i]=G_bar[j]; G_bar[j]=tmp;} while(false);
 	}
 
 	void reconstruct_gradient()
@@ -761,7 +761,7 @@ class Solver {
 			}
 		}
 
-		if(Gmax+Gmax2 < eps)
+		if(Gmax+Gmax2 < eps || Gmin_idx == -1)
 			return 1;
 
 		working_set[0] = Gmax_idx;
@@ -1007,7 +1007,7 @@ final class Solver_NU extends Solver
 			}
 		}
 
-		if(Math.max(Gmaxp+Gmaxp2,Gmaxn+Gmaxn2) < eps)
+		if(Math.max(Gmaxp+Gmaxp2,Gmaxn+Gmaxn2) < eps || Gmin_idx == -1)
 			return 1;
 	
 		if(y[Gmin_idx] == +1)
@@ -1182,8 +1182,8 @@ class SVC_Q extends Kernel
 	{
 		cache.swap_index(i,j);
 		super.swap_index(i,j);
-		do {byte _=y[i]; y[i]=y[j]; y[j]=_;} while(false);
-		do {double _=QD[i]; QD[i]=QD[j]; QD[j]=_;} while(false);
+		do {byte tmp=y[i]; y[i]=y[j]; y[j]=tmp;} while(false);
+		do {double tmp=QD[i]; QD[i]=QD[j]; QD[j]=tmp;} while(false);
 	}
 }
 
@@ -1222,7 +1222,7 @@ class ONE_CLASS_Q extends Kernel
 	{
 		cache.swap_index(i,j);
 		super.swap_index(i,j);
-		do {double _=QD[i]; QD[i]=QD[j]; QD[j]=_;} while(false);
+		do {double tmp=QD[i]; QD[i]=QD[j]; QD[j]=tmp;} while(false);
 	}
 }
 
@@ -1259,9 +1259,9 @@ class SVR_Q extends Kernel
 
 	void swap_index(int i, int j)
 	{
-		do {byte _=sign[i]; sign[i]=sign[j]; sign[j]=_;} while(false);
-		do {int _=index[i]; index[i]=index[j]; index[j]=_;} while(false);
-		do {double _=QD[i]; QD[i]=QD[j]; QD[j]=_;} while(false);
+		do {byte tmp=sign[i]; sign[i]=sign[j]; sign[j]=tmp;} while(false);
+		do {int tmp=index[i]; index[i]=index[j]; index[j]=tmp;} while(false);
+		do {double tmp=QD[i]; QD[i]=QD[j]; QD[j]=tmp;} while(false);
 	}
 
 	float[] get_Q(int i, int len)
@@ -1293,7 +1293,7 @@ public class svm {
 	//
 	// construct and solve various formulations
 	//
-	public static final int LIBSVM_VERSION=319; 
+	public static final int LIBSVM_VERSION=322; 
 	public static final Random rand = new Random();
 
 	private static svm_print_interface svm_print_stdout = new svm_print_interface()
@@ -1751,7 +1751,7 @@ public class svm {
 		for(i=0;i<prob.l;i++)
 		{
 			int j = i+rand.nextInt(prob.l-i);
-			do {int _=perm[i]; perm[i]=perm[j]; perm[j]=_;} while(false);
+			do {int tmp=perm[i]; perm[i]=perm[j]; perm[j]=tmp;} while(false);
 		}
 		for(i=0;i<nr_fold;i++)
 		{
@@ -1899,8 +1899,8 @@ public class svm {
 		//
 		if (nr_class == 2 && label[0] == -1 && label[1] == +1)
 		{
-			do {int _=label[0]; label[0]=label[1]; label[1]=_;} while(false);
-			do {int _=count[0]; count[0]=count[1]; count[1]=_;} while(false);
+			do {int tmp=label[0]; label[0]=label[1]; label[1]=tmp;} while(false);
+			do {int tmp=count[0]; count[0]=count[1]; count[1]=tmp;} while(false);
 			for(i=0;i<l;i++)
 			{
 				if(data_label[i] == 0)
@@ -2202,7 +2202,7 @@ public class svm {
 				for(i=0;i<count[c];i++)
 				{
 					int j = i+rand.nextInt(count[c]-i);
-					do {int _=index[start[c]+j]; index[start[c]+j]=index[start[c]+i]; index[start[c]+i]=_;} while(false);
+					do {int tmp=index[start[c]+j]; index[start[c]+j]=index[start[c]+i]; index[start[c]+i]=tmp;} while(false);
 				}
 			for(i=0;i<nr_fold;i++)
 			{
@@ -2234,7 +2234,7 @@ public class svm {
 			for(i=0;i<l;i++)
 			{
 				int j = i+rand.nextInt(l-i);
-				do {int _=perm[i]; perm[i]=perm[j]; perm[j]=_;} while(false);
+				do {int tmp=perm[i]; perm[i]=perm[j]; perm[j]=tmp;} while(false);
 			}
 			for(i=0;i<=nr_fold;i++)
 				fold_start[i]=i*l/nr_fold;
@@ -2428,7 +2428,13 @@ public class svm {
 					pairwise_prob[j][i]=1-pairwise_prob[i][j];
 					k++;
 				}
-			multiclass_probability(nr_class,pairwise_prob,prob_estimates);
+			if (nr_class == 2)
+			{
+				prob_estimates[0] = pairwise_prob[0][1];
+				prob_estimates[1] = pairwise_prob[1][0];
+			}
+			else
+				multiclass_probability(nr_class,pairwise_prob,prob_estimates);
 
 			int prob_max_idx = 0;
 			for(i=1;i<nr_class;i++)
@@ -2549,6 +2555,11 @@ public class svm {
 	{
 		svm_parameter param = new svm_parameter();
 		model.param = param;
+		// parameters for training only won't be assigned, but arrays are assigned as NULL for safety
+		param.nr_weight = 0;
+		param.weight_label = null;
+		param.weight = null;
+
 		try
 		{
 			while(true)
